#' Case-mix Adjust Item Scores
#'
#' This function adjusts group-level item scores for its case mix, or the characteristics that make up the group, using a linear model approach.
#'
#' @param item A string representing the name of the item variable to be adjusted.
#' @param adjusters A character vector of variable names to be used as case-mix adjusters in the model. Defaults to `NULL`.
#' @param data A data frame containing the dataset.
#' @param group A string representing the name of the group or unit variable. This is the level at which scores will be reported.
#' @param type A string representing the name of the type variable, which may describe units of different types. Defaults to `NULL`.
#' @param weight A string representing the name of the weight variable, if groups are weighted. Individual weights should not be used. Defaults to `NULL`.
#' @param total_weight A string representing the name of the total weight variable, if groups are weighted. Defaults to `NULL`.
#' @param una_mean A string representing the name of the unadjusted mean variable, if unadjusted means are pre-computed and do not require computation in this dataset. Defaults to `NULL`.
#' @param prev_coef A data frame containing previous coefficients for adjustment. This allows for more efficient computation or applying case-mix adjustment to a new population. Defaults to `NULL`.
#' @param natl_means A data frame containing national means for adjusters,  if these are pre-computed. Defaults to `NULL`.
#' @param id A string representing the name of the individual-level ID variable. Defaults to `NULL`.
#' @param write_folder A string representing the directory path to write output files. Defaults to `NULL`.
#' @param exclude A logical value indicating whether to exclude units with small samples. Defaults to `TRUE`.
#' @param enforce_boundaries A logical value indicating whether to enforce boundaries on adjusted means. Defaults to `TRUE`.
#'
#' @return A data frame summarizing the adjusted means, variances, and other statistics for each group.
#'
#' @details The function performs several steps including data cleaning, calculation of unadjusted means, running a linear model, and calculating adjusted means and variances. It also writes intermediate data to files if a `write_folder` is specified.
#'
#' @return A data frame with the following variables:
#' \describe{
#'   \item{group}{A string representing the name of the group or unit variable. This is the level at which scores will be reported. This column will be named whatever is passed in for the `group` argument.}
#'   \item{type}{A string representing type of group, if applicable.}
#'   \item{item}{A string representing the name of the item.}
#'   \item{alln}{Total group sample size.}
#'   \item{usen}{Sample size of the item.}
#'   \item{plan_weight}{Weight associated with each group/type.}
#'   \item{una_mean}{Unadjusted mean score. If group weights are specified, this is equivalent to a weighted mean.}
#'   \item{uwt_mean}{Unweighted mean score, equivalent to an arithmetic mean.}
#'   \item{adj_mean}{Case-mix adjusted mean score.}
#'   \item{sas_vp}{Within group residual variance.}
#'   \item{d_pj}{Weighted deviation for each observation from the group mean, wrapped up in a tibble with the following:}
#'   \itemize{
#'     \item{sas_dpj}{Deviation of observation-level residual from the its within-group mean.}
#'     \item{u_pj}{Weight attached to each observation.}
#'   }
#'   \item{total_weight}{Total weight associated with a group.}
#'   \item{vi}{Within-group variance component.}
#'   \item{vo}{Between-group variance component.}
#'   \item{var_deltahat}{Total variance comprised of the summation of `vi` and `vo`.}
#'   \item{delta}{Difference between `adj_mean` and `grand_mean`.}
#'   \item{casemix_coefs}{Adjusted casemix coefficients.}
#'   \item{grand_mean}{Overall mean across all groups and types.}
#'   \item{item_min}{Minimum value of the variable `item`.}
#'   \item{item_max}{Maximum value of the variable `item`.}
#'   \item{casemix_coefs}{List column of ase-mix coefficients.}
#' }
#'
#' @import dplyr
#' @importFrom tidyr gather spread
#'
#' @examples
#' n_grps <- 10
#' group_ds <- tibble::tibble(grp_id = 1:n_grps,
#'                            grp_quality = rnorm(n_grps, sd = 2),
#'                            grp_n = rpois(n_grps, 5)*75 + 30)
#' nn <- sum(group_ds$grp_n)
#' individual_ds <- tibble::tibble(individual_id = 1:nn,
#'                                 grp_id = rep(group_ds$grp_id, group_ds$grp_n),
#'                                 x1 = rnorm(nn),
#'                                 x2 = rnorm(nn),
#'                                 x3 = 1*(rnorm(nn) > 1))
#' analysis_ds <- dplyr::inner_join(group_ds, individual_ds) |>
#'   dplyr::mutate(y = grp_quality + x1 - x2 - 3*x3 + rnorm(nn, sd = 0.5))
#'
#' adj_item(item = 'y', adjusters = c('x1', 'x2', 'x3'), data = analysis_ds, group = 'grp_id', id = 'individual_id', enforce_boundaries = FALSE)
#'
#' @export
adj_item <- function(item,
                     adjusters = NULL,
                     data,
                     group,
                     type = NULL,
                     weight = NULL,
                     total_weight = NULL,
                     una_mean = NULL,
                     prev_coef = NULL,
                     natl_means = NULL,
                     id = NULL,
                     write_folder = NULL,
                     exclude = TRUE,
                     enforce_boundaries = TRUE) {
  ###############################
  ## Data cleaning
  ###############################
    clean_data <- prep_for_adj(
      data = data,
      item = item,
      group = group,
      weight = weight,
      exclude = exclude,
      id = id
    )

  item_sum <- clean_data |>
    summarise(item_max = max(!!sym(item)),
              item_min = min(!!sym(item)))
  if (!is.null(write_folder)) {
    write_csv(clean_data, glue::glue('{write_folder}/clean-data_item-{item}.csv'))
  }
  if (is.null(weight))
    weight <- 'wt'
  if(is.null(type)) {
    type <- 'type'
    clean_data <- clean_data |>
      mutate(!!type := '')
  }

  ###############################
  ## Calculate some facts about the unit:
  ##    unadjusted mean, usen, total weight
  ###############################
  if (is.null(total_weight) & is.null(una_mean)) {
    plan_facts <- clean_data |>
      group_by(!!sym(group), !!sym(type)) |>
      summarise(una_mean = sum(!!sym(item) * !!sym(weight)) / sum(!!sym(weight)),
                total_weight = sum(!!sym(weight)),
                uwt_mean = mean(!!sym(item)),
                alln = unique(alln),
                usen = n()) |>
      ungroup()
  } else if (!is.null(total_weight) & is.null(una_mean)) {
    plan_facts <- clean_data |>
      group_by(!!sym(group), !!sym(type)) |>
      summarise(una_mean = sum(!!sym(item) * !!sym(weight)) / sum(!!sym(weight)),
                total_weight = unique(!!sym(total_weight)),
                uwt_mean = mean(!!sym(item)),
                alln = unique(alln),
                usen = n()) |>
      ungroup()
  } else if (!is.null(una_mean) & is.null(total_weight)) {
    plan_facts <- clean_data |>
      group_by(!!sym(group), !!sym(type)) |>
      summarise(una_mean = unique(!!sym(una_mean)),
                total_weight = sum(!!sym(weight)),
                uwt_mean = mean(!!sym(item)),
                alln = unique(alln),
                usen = n()) |>
      ungroup()
  } else {
    plan_facts <- clean_data |>
      group_by(!!sym(group), !!sym(type)) |>
      summarise(una_mean = unique(!!sym(una_mean)),
                total_weight = unique(!!sym(total_weight)),
                uwt_mean = mean(!!sym(item)),
                alln = unique(alln),
                usen = n()) |>
      ungroup()
  }

  grand_mean <- plan_facts |>
    summarise(sum(una_mean*total_weight)/sum(total_weight)) |>
    unlist()

  if (is.null(adjusters)) {
    clean_data <- clean_data |>
      mutate(intercept_ind = 1)
    adjusters <- 'intercept_ind'
  }

  ###################################
  ## Create data for linear model by de-meaning outcome and adjusters within unit/group
  ###################################

  ######################################
  ## Run linear model
  ######################################
    model_data <- clean_data |>
      group_by(!!sym(group)) |>
      mutate_at(vars(c(!!sym(item), adjusters)), ~ (. - mean(.))) |>
      ungroup()
    if (!is.null(write_folder)) {
      write_csv(model_data, glue::glue('{write_folder}/model-data_item-{item}.csv'))
    }
  if (is.null(prev_coef)) {
    model_fm <-
      glue::glue('{item} ~ {paste0(adjusters, collapse = " + ")}')
    mod <-
      lm(as.formula(model_fm),
         data = model_data,
         weights = model_data |> pull(!!sym(weight)))
    casemix_coefs <- broom::tidy(mod) |>
      mutate(item = item)
    if (!is.null(write_folder)) {
      write_rds(mod, glue::glue('{write_folder}/casemix-adjustment-model_{item}.rds'))
    }
  } else if (is.data.frame(prev_coef)) {
    casemix_coefs <- prev_coef
  } else stop('Casemix coefficients should be in data.frame format...')



  #####################################
  ## Get predicted value from model at:
  ##    difference between plan mean of case-mix adjusters and overall means
  #####################################
  if (is.null(natl_means)) {
    natl_means <- clean_data |>
      group_by(!!sym(group)) |> summarise_at(vars(adjusters),
                                              ~mean(.)) |>
      inner_join(plan_facts, by = group) |>
      mutate_at(vars(adjusters), ~(sum(. * total_weight)/sum(total_weight))) |>
      select(!!sym(group), all_of(adjusters)) |>
      distinct() |>
      gather(var, natl_mean, -!!sym(group))
  }
  if (!is.null(write_folder)) {
    write_csv(natl_means, glue::glue('{write_folder}/natl-means_item-{item}.csv'))
  }
  plan_means <- clean_data |>
    group_by(!!sym(group)) |> summarise_at(vars(adjusters),
                                            ~mean(.)) |>
    gather(var, plan_mean, -!!sym(group))
  cma_vals_for_prediction <- natl_means |>
    inner_join(plan_means) |>
    transmute(!!sym(group), var, cma_val = natl_mean - plan_mean) |>
    spread(var, cma_val)
  if (!is.null(write_folder)) {
    write_csv(cma_vals_for_prediction, glue::glue('{write_folder}/cma-vals-for-prediction_item-{item}.csv'))
  }

  ###############################################
  ## Get predicted values and add to unadjusted mean to get adjusted score
  ###############################################
  predicted_vals <- cma_vals_for_prediction |>
    inner_join(plan_facts, by = group) |>
    mutate(
      model_resid = predict_light(data = cma_vals_for_prediction, adjusters = adjusters, coefs = casemix_coefs$estimate),
      adj_mean = una_mean + model_resid,
      adj_mean = adj_mean - sum(total_weight*adj_mean)/sum(total_weight) + sum(total_weight*una_mean)/sum(total_weight)
    )
  if (!is.null(write_folder)) {
    write_csv(predicted_vals, glue::glue('{write_folder}/predicted-values_item-{item}.csv'))
  }
  ####################################
  ## Calculate variance
  ####################################
  variance_parts <- clean_data |>
    mutate(
      y = model_data |> pull(!!sym(item)),
      z_pj = y - predict_light(data = model_data, adjusters = adjusters, coefs = casemix_coefs$estimate)
    ) |>
    group_by(!!sym(group), !!sym(type)) |>
    mutate(
      n_p = n(),
      u_pj = !!sym(weight),
      mu_p = sum(u_pj * z_pj) / sum(u_pj), ## u_pj * z_pj = w_yresid in SAS; this numerator is called sum_w_yresid and the denominator is called sum_wgtresp
      d_pj = u_pj * z_pj / sum(u_pj),
      d_pj = d_pj - sum(u_pj * d_pj)/sum(u_pj), ## take out the weighted within-plan mean
      sas_dpj = z_pj - mu_p, ## std_yresid in SAS
      sas_dpj_sq = sas_dpj^2, ## sq_std_yresid
      sas_wt_sq = sum(u_pj)^2, ## sq_wgtresp
    ) |>
    select(!!sym(id), z_pj, n_p, u_pj, mu_p, d_pj, sas_dpj, sas_dpj_sq, sas_wt_sq)
  if (!is.null(write_folder)) {
    write_csv(variance_parts, glue::glue('{write_folder}/variance-parts_item-{item}.csv'))
  }
  variances <- variance_parts |>
    group_by(!!sym(group), !!sym(type)) |>
    summarise(sas_vp = sum(sas_dpj_sq/sas_wt_sq*u_pj^2),
              d_pj = list(tibble(!!sym(id), d_pj, sas_dpj, u_pj))) |>
    ungroup() |>
    inner_join(plan_facts |> select(!!sym(group), total_weight)) |>
    mutate(
      vi = (1 - 2*total_weight/sum(total_weight))*sas_vp,
      vo = (sum(total_weight))^(-2)*sum(total_weight^2 * sas_vp),
      var_deltahat = vi + vo
    )


  group_summary <- predicted_vals |>
    group_by(!!sym(group), !!sym(type)) |>
    summarise(
      item = item,
      alln = mean(alln),
      usen = mean(usen),
      plan_weight = mean(total_weight),
      una_mean = mean(una_mean),
      uwt_mean = mean(uwt_mean, na.rm=TRUE),
      # model_resid = mean(model_resid),
      adj_mean = mean(adj_mean)) |>
    inner_join(variances, by = c(group, type)) |>
    mutate(delta = adj_mean - grand_mean,
           casemix_coefs = list(casemix_coefs),
           grand_mean = grand_mean,
           item_min = item_sum$item_min,
           item_max = item_sum$item_max) |>
    ungroup()
  if (enforce_boundaries) {
    group_summary <- group_summary |>
      mutate(adj_mean = pmax(item_sum$item_min, pmin(item_sum$item_max, adj_mean)))
  }
  if (any(count(group_summary, !!sym(group))$n > 1)) browser()

  group_summary

}
