#' Perform Case-mix Adjustment for Composite Scores
#'
#' This function computes case-mix adjusted composite scores from multiple items, adjusting for group characteristics.
#'
#' @param data A data frame containing the dataset.
#' @param group A string representing the name of the unit/grouping variable.
#' @param id A string representing the name of the individual-level  ID variable.
#' @param adjvars A character vector of variable names to be used as case-mix adjusters in the model.
#' @param weight A string representing the name of the weight variable. Defaults to `NULL`.
#' @param type A string representing the name of the type variable, which may describe units of different types. Defaults to `NULL`.
#' @param items A character vector of item names to be included in the composite.
#' @param composite_name A string representing the name of the composite score.
#' @param k An integer specifying the that items with sample size lower than `k` should receive lower weight. Defaults to 1.
#' @param prev_coef_comp A data frame containing previous coefficients for composite adjustment. Defaults to `NULL`.
#' @param write_folder A string representing the directory path to write output files. Defaults to `NULL`.
#'
#' @return A data frame with the following variables:
#' \describe{
#'   \item{group}{A string representing the name of the group or unit variable. This is the level at which scores will be reported. This column will be named whatever is passed in for the `group` argument.}
#'   \item{plan_weight}{Average weight associated with each group/type.}
#'   \item{raw_una_mean}{Weighted average of unadjusted composite scores.}
#'   \item{una_mean}{Unadjusted mean of the composite score for each group and type.}
#'   \item{uwt_mean}{Unweighted mean of the composite score for each group and type.}
#'   \item{adj_mean}{Adjusted mean for each group and type.}
#'   \item{delta}{Difference between adjusted mean and mean including all groups (`grand_mean`).}
#'   \item{item_min}{Minimum value of the variable `item`.}
#'   \item{item_max}{Maximum value of the variable `item`.}
#'   \item{alln}{Total group sample size.}
#'   \item{usen}{Sample size of the measure/item.}
#'   \item{sas_vp}{Within-group residual variance of the adjusted mean.}
#'   \item{all_plan_weight}{Summation of the plan weights.}
#'   \item{all_vp}{Total within-group variability across all groups.}
#'   \item{vi}{Within-group variance component.}
#'   \item{vo}{Between-group variance component.}
#'   \item{var_deltahat}{Total variance of `delta` comprised of the summation of `vi` and `vo`.}
#'   \item{grand_mean}{Mean including all groups.}
#'   \item{item}{A string representing the name of the composite.}
#'   \item{components}{A character vector of item names comprising the composite.}
#' }
#'
#' @details The function prepares data, computes unadjusted and adjusted means, and calculates composite scores using specified items and adjusters. It also handles weighting and variance calculations.
#'
#' @import rlang
#' @importFrom purrr map_dfr
#' @importFrom tidyr unnest
#'
#' @examples
#' n_grps <- 10
#' group_ds <- tibble::tibble(grp_id = 1:n_grps,
#'                            grp_quality = rnorm(n_grps, sd = 2),
#'                            grp_n = rpois(n_grps, 5)*75 + 30)
#' nn <- sum(group_ds$grp_n)
#' individual_ds <- tibble::tibble(individual_id = 1:nn,
#'                                 grp_id = rep(group_ds$grp_id, group_ds$grp_n),
#'                                 x1 = rnorm(nn),
#'                                 x2 = rnorm(nn),
#'                                 x3 = 1*(rnorm(nn) > 1))
#' analysis_ds <- dplyr::inner_join(group_ds, individual_ds) |>
#'   dplyr::mutate(y1 = grp_quality + x1 - x2 - 3*x3 + rnorm(nn, sd = 0.5),
#'                 y2 = grp_quality/2 + 0.5*x1 - 2*x2 - x3 + rnorm(nn, sd = 0.5))
#'
#' compute_composite(data = analysis_ds, group = 'grp_id', id = 'individual_id', adjvars = c('x1', 'x2', 'x3'), items = c('y1', 'y2'), composite_name = 'my_composite')
#'
#' @export
compute_composite <- function(data, group, id, adjvars, weight = NULL, type = NULL, items, composite_name, k = 1, prev_coef_comp = NULL, write_folder = NULL) {
  item_data <- prep_for_adj(data = data, item = items, group = group, id = id, weight = weight)

  ns <- distinct(item_data, !!sym(group), alln, usen)
  if (is.null(type)) {
    item_data$type <- ''
    data$type <- ''
    type <- 'type'
  }
  if (is.null(weight)) {
    weight <- 'wt'
  }
  item_wts <- item_data |>
    count(!!sym(group), !!sym(type), item) |>
    mutate(wtn = pmin(n, k)) |>
    group_by(!!sym(group), !!sym(type)) |>
    mutate(item_wt = wtn/sum(wtn)) |>
    ungroup()
  grp_wts <- item_data |>
    group_by(item, !!sym(group)) |>
    summarise(total_weight = sum(!!sym(weight))) |>
    filter(total_weight > 0)

  unadjusted_means <- item_data |>
    group_by(item, !!sym(group)) |>
    filter(sum(!!sym(weight)) > 0) |>
    summarise(unadj_wt_mean = sum(y*!!sym(weight))/sum(!!sym(weight)))
  item_national_means <- unadjusted_means |>
    group_by(item) |>
    summarise(natl_mean = mean(unadj_wt_mean)) |>
    ungroup()
  
  adjustment_terms <- unadjusted_means |>
    inner_join(item_wts) |>
    inner_join(item_national_means) |>
    group_by(!!sym(group)) |>
    summarise(composite_adj_term = sum(item_wt*(unadj_wt_mean - natl_mean)))
  national_composite_mean <- unadjusted_means |>
    group_by(item) |>
    summarise(unadj_wt_mean = mean(unadj_wt_mean)) |>
    ungroup() |>
    summarise(unadj_mean = mean(unadj_wt_mean))
  una_means <- adjustment_terms |>
    mutate(comp_una_mean = composite_adj_term + national_composite_mean$unadj_mean)

  data_for_adjustment <- item_data |>
    inner_join(grp_wts, join_by(item, !!sym(group))) #|>
  # inner_join(una_means)

  adjusted_data <- map_dfr(items, function(i) {
    this_data <- data_for_adjustment |> filter(item == i)
    if (is.null(prev_coef_comp)) {prev_coef = NULL}
    else {prev_coef <- prev_coef_comp |> filter(item == i)}
    adj_item(item = i, adjusters = adjvars, data = this_data, group = group, type = type, weight = weight, id = id, total_weight = 'total_weight', prev_coef = prev_coef, write_folder = write_folder, exclude = FALSE, enforce_boundaries = FALSE)
  })


  comp_res_mean <- adjusted_data |>
    inner_join(item_wts) |>
    inner_join(una_means) |>
    group_by(!!sym(group), !!sym(type)) |>
    summarise(plan_weight = sum(plan_weight),
              raw_una_mean = wtd_mean(una_mean, item_wt),
              una_mean = mean(comp_una_mean),
              uwt_mean = mean(uwt_mean, na.rm=TRUE),
              adj_mean = mean(wtd_mean(adj_mean, item_wt) - raw_una_mean + una_mean),
              delta = wtd_mean(delta, item_wt),
              item_min = wtd_mean(item_min, item_wt),
              item_max = wtd_mean(item_max, item_wt)) |>
    ungroup() |>
    mutate(
      una_mean = pmax(item_min, pmin(item_max, una_mean)),
      adj_mean = pmax(item_min, pmin(item_max, adj_mean)))

  comp_vp <- adjusted_data |>
    inner_join(item_wts) |>
    select(item, !!sym(group), !!sym(type), d_pj, total_weight, item_wt) |>
    unnest(d_pj) |>
    group_by(item, !!sym(group), !!sym(type)) |>
    mutate(resid_term = item_wt*sas_dpj/total_weight) |>
    group_by(!!sym(id), !!sym(group), !!sym(type), u_pj) |>
    summarize(resid_term_by_id = sum(resid_term)) |>
    mutate(vp_wt = u_pj**2*resid_term_by_id**2) |>
    group_by(!!sym(group), !!sym(type)) |>
    summarize(sas_vp = sum(vp_wt))

  ttest_prep <- comp_res_mean |>
    inner_join(ns) |>
    inner_join(comp_vp, by = join_by(!!sym(group), !!sym(type))) |>
    mutate(all_plan_weight = sum(plan_weight),
           all_vp = sum(sas_vp * plan_weight**2),
           vi = sas_vp*(1 - (2*plan_weight)/all_plan_weight),
           vo = all_vp/all_plan_weight**2,
           var_deltahat = vi + vo,
           grand_mean = median(adj_mean - delta),
           item = composite_name,
           components = paste0(items, collapse = ', '))
  ttest_prep
}
