#' Lightweight Prediction Using External Coefficients
#'
#' This function performs predictions using a linear model approach with provided coefficients.
#'
#' @param data A data frame containing the dataset.
#' @param fm An optional formula specifying the model. Defaults to `NULL`.
#' @param adjusters A character vector of variable names to be used as adjusters in the model. Defaults to `NULL`.
#' @param coefs A numeric vector of coefficients for the model.
#'
#' @return A numeric vector of predicted values.
#'
#' @details The function constructs a model matrix using the specified adjusters and applies the provided coefficients to generate predictions. If any coefficients are missing, they are set to zero with a warning.
#'
#' @examples
#' \dontrun{
#' predict_light(data = my_data, adjusters = c("var1", "var2"), coefs = c(-1, 0.5, 1.2))
#' }
#'
#' @export
predict_light <- function(data, fm = NULL, adjusters = NULL, coefs) {
  data <- data |>
    mutate(fakey = 1)
  if (is.null(fm)) {
    fm <- as.formula(glue::glue('fakey ~ {paste0(adjusters, collapse = " + ")}'))
  }
  if (any(is.na(coefs))) {
    warning("Some coefficients missing. Set to 0.")
    coefs <- ifelse(is.na(coefs), 0, coefs)
  }
  mod_mat <- model.matrix(fm, data)
  as.vector(mod_mat %*% coefs)
}
